/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "hostentry.h"

namespace Conexus
{
  namespace Gtk
  {

    HostEntry::HostEntry ( unsigned lookup ) :
        ::Gtk::VBox(),
        m_address ( NULL ),
        m_lookup ( lookup ),
        m_immediate_apply ( true ),
        m_apply_button ( ::Gtk::Stock::APPLY )
    {
      this->setup_widget();
    }

    HostEntry::HostEntry ( IPAddressInterface* address, unsigned lookup ) :
        ::Gtk::VBox(),
        m_address ( address ),
        m_lookup ( lookup ),
        m_immediate_apply ( true ),
        m_apply_button ( ::Gtk::Stock::APPLY )
    {
      this->setup_widget();
    }

    HostEntry::HostEntry ( IPAddressInterface::pointer address, unsigned lookup ) :
        ::Gtk::VBox(),
        m_address ( address.get() ),
        m_lookup ( lookup ),
        m_immediate_apply ( true ),
        m_apply_button ( ::Gtk::Stock::APPLY )
    {
      this->setup_widget();
    }

    HostEntry::HostEntry ( IPv4::Address& address, unsigned lookup ) :
        ::Gtk::VBox(),
        m_address ( &address ),
        m_lookup ( lookup ),
        m_immediate_apply ( true ),
        m_apply_button ( ::Gtk::Stock::APPLY )
    {
      this->setup_widget();
    }

    HostEntry::HostEntry ( IPv6::Address& address, unsigned lookup ) :
        ::Gtk::VBox(),
        m_address ( &address ),
        m_lookup ( lookup ),
        m_immediate_apply ( true ),
        m_apply_button ( ::Gtk::Stock::APPLY )
    {
      this->setup_widget();
    }

    HostEntry::~HostEntry() {}

    IPAddressInterface * HostEntry::address()
    {
      return m_address;
    }

    void HostEntry::set_address ( IPAddressInterface* addr, unsigned lookup )
    {
      if ( m_conn_address_host ) m_conn_address_host.disconnect();
      m_lookup = lookup;
      m_address = addr;
      if ( m_address == NULL ) return;
      m_conn_address_host = m_address->signal_host_changed().connect ( sigc::mem_fun ( *this, &HostEntry::on_controlled_host_changed ) );
      this->on_controlled_host_changed();
    }

    void HostEntry::set_address ( IPAddressInterface::pointer addr, unsigned lookup )
    {
      this->set_address( addr.get(), lookup );
    }
    
    void HostEntry::set_address ( IPv4::Address & addr, unsigned lookup )
    {
      this->set_address( &addr, lookup );
    }

    void HostEntry::set_address ( IPv6::Address & addr, unsigned lookup )
    {
      this->set_address( &addr, lookup );
    }

    void HostEntry::unset_address()
    {
      if ( m_conn_address_host ) m_conn_address_host.disconnect();
      m_lookup = LOOKUP_NONE;
      m_address = NULL;
      this->set_host ( "" );
    }

    ::Gtk::ComboBoxEntryText& HostEntry::host_widget( )
    {
      return m_host_widget;
    }

    ::Gtk::Image & HostEntry::good_host_image( )
    {
      return m_good_host_image;
    }

    ::Gtk::Image & HostEntry::bad_host_image( )
    {
      return m_bad_host_image;
    }

    std::string HostEntry::host( )
    {
      return m_host_widget.get_entry()->get_text();
    }

    void HostEntry::set_host ( const std::string& s )
    {
      m_host_widget.prepend_text ( s );
      m_host_widget.set_active_text ( s );
    }

    bool HostEntry::is_hostname_label_visible()
    {
      return m_hostname_label.is_visible();
    }

    bool HostEntry::is_host_label_visible()
    {
      return m_host_label.is_visible();
    }

    bool HostEntry::is_host_image_visible()
    {
      return m_image_holder.is_visible();
    }

    void HostEntry::show_hostname_label ( bool show )
    {
      if ( show )
        m_hostname_label.show_all();
      else
        m_hostname_label.hide();
    }

    void HostEntry::show_host_label ( bool show )
    {
      if ( show )
        m_host_label.show_all();
      else
        m_host_label.hide();
    }

    void HostEntry::show_host_image ( bool show )
    {
      if ( show )
        m_image_holder.show_all();
      else
        m_image_holder.hide();
    }

    ::Gtk::Button & HostEntry::apply_button()
    {
      return m_apply_button;
    }

    bool HostEntry::is_immediate_apply_enabled()
    {
      return m_immediate_apply;
    }

    void HostEntry::enable_immediate_apply ( bool enable )
    {
      m_immediate_apply = enable;

      if ( enable )
      {
        m_apply_button.hide();
        this->apply();
      }
      else
      {
        m_apply_button.show();
      }
    }

    void HostEntry::apply()
    {
      std::string new_hostname;
      std::string new_host;
      std::string temp_host = this->host();
      bool valid = false;

      m_host_widget_connection.block();

      if ( ( m_lookup & LOOKUP_IPV4 ) and IPv4::is_valid_hostname ( temp_host ) )
      {
        uint32_t h = IPv4::hostname_to_host( temp_host );
        new_hostname = IPv4::host_to_hostname( h );
        new_host = IPv4::host_to_string( h );
        valid = true;
      }
      
      if ( ( m_lookup & LOOKUP_IPV6 ) and IPv6::is_valid_hostname ( temp_host ) )
      {
        in6_addr h = IPv6::hostname_to_host( temp_host );
        new_hostname = IPv6::host_to_hostname( h );
        new_host = IPv6::host_to_string( h );
        valid = true;
      }

      if ( valid )
      {
        if ( m_image_holder.get_child() != &m_good_host_image )
        {
          m_image_holder.remove();
          m_image_holder.add ( m_good_host_image );
          m_image_holder.show_all_children();
        }

        m_host_label.set_markup ( new_host );

        m_hostname_label.set_markup ( new_hostname );

        m_apply_button.set_sensitive ( false );

        m_host_widget.remove_text( temp_host );
        m_host_widget.prepend_text( temp_host );
        
        if ( m_address ) m_address->set_host ( temp_host );
      }
      else
      {
        if ( m_image_holder.get_child() != &m_bad_host_image )
        {
          m_image_holder.remove();
          m_image_holder.add ( m_bad_host_image );
          m_hostname_label.set_markup ( "<small><i>Invalid Entry</i></small>" );
          m_host_label.set_markup ( "<small><i>Invalid Entry</i></small>" );
//           m_signal_host_changed.emit ( false );
        }
      }

      m_host_widget_connection.unblock();

    }

    void HostEntry::on_controlled_host_changed()
    {
    }

    void HostEntry::on_host_widget_changed( )
    {
      if ( m_immediate_apply ) this->apply();
      else m_apply_button.set_sensitive(true);
    }

    void HostEntry::setup_widget( )
    {
      set_homogeneous ( false );

      ::Gtk::HBox* hbox = ::Gtk::manage ( new ::Gtk::HBox() );
      hbox->set_homogeneous ( false );

      m_apply_button.hide();
      m_apply_button.set_no_show_all();
      m_apply_button.set_sensitive(false);

      hbox->pack_start( m_host_widget );
      hbox->pack_start( m_image_holder, ::Gtk::PACK_SHRINK );
      hbox->pack_start( m_apply_button );

      m_host_widget_connection = m_host_widget.signal_changed().connect ( sigc::mem_fun ( *this, &HostEntry::on_host_widget_changed ) );

      m_good_host_image.set_from_icon_name ( "stock_internet", ::Gtk::IconSize ( ::Gtk::ICON_SIZE_LARGE_TOOLBAR ) );
      m_bad_host_image.set_from_icon_name ( "gtk-dialog-error", ::Gtk::IconSize ( ::Gtk::ICON_SIZE_LARGE_TOOLBAR ) );
      m_image_holder.add ( m_bad_host_image );

      pack_start ( *hbox );

      m_hostname_label.set_markup ( "<small><i>Invalid Entry</i></small>" );
      m_host_label.set_markup ( "<small><i>Invalid Entry</i></small>" );
      m_hostname_label.set_use_markup ( true );
      m_host_label.set_use_markup ( true );

      pack_start ( m_hostname_label );
      pack_start ( m_host_label );

      show_all_children();
    }

  }

}

