/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "portentry.h"

#include <sstream>

#define DEFAULT_PORT 1
#define MIN_PORT 0
#define MAX_PORT 32767
#define PORT_INC 1
#define PORT_PAGE_INC 100

namespace Conexus
{
  namespace Gtk
  {

    PortEntry::PortEntry ( unsigned lookup ) :
        ::Gtk::VBox(),
        m_address ( NULL ),
        m_lookup ( lookup ),
        m_immediate_apply ( true ),
        m_port_widget_adjustment ( DEFAULT_PORT, MIN_PORT, MAX_PORT, PORT_INC, PORT_PAGE_INC ),
        m_apply_button( ::Gtk::Stock::APPLY )
    {
      this->setup_widget();
    }

    PortEntry::PortEntry ( IPAddressInterface* addr, unsigned lookup ) :
        ::Gtk::VBox(),
        m_address ( addr ),
        m_lookup ( lookup ),
        m_immediate_apply ( true ),
        m_port_widget_adjustment ( DEFAULT_PORT, MIN_PORT, MAX_PORT, PORT_INC, PORT_PAGE_INC ),
        m_apply_button( ::Gtk::Stock::APPLY )
    {
      this->setup_widget();
    }

    PortEntry::PortEntry ( IPAddressInterface::pointer addr, unsigned lookup ) :
        ::Gtk::VBox(),
        m_address ( addr.get() ),
        m_lookup ( lookup ),
        m_immediate_apply ( true ),
        m_port_widget_adjustment ( DEFAULT_PORT, MIN_PORT, MAX_PORT, PORT_INC, PORT_PAGE_INC ),
        m_apply_button( ::Gtk::Stock::APPLY )
    {
      this->setup_widget();
    }

    PortEntry::PortEntry ( IPv4::Address & addr, unsigned lookup ) :
        ::Gtk::VBox(),
        m_address ( &addr ),
        m_lookup ( lookup ),
        m_port_widget_adjustment ( DEFAULT_PORT, MIN_PORT, MAX_PORT, PORT_INC, PORT_PAGE_INC ),
        m_apply_button( ::Gtk::Stock::APPLY )
    {
      this->setup_widget();
    }

    PortEntry::PortEntry ( IPv6::Address & addr, unsigned lookup ) :
        ::Gtk::VBox(),
        m_address ( &addr ),
        m_lookup ( lookup ),
        m_port_widget_adjustment ( DEFAULT_PORT, MIN_PORT, MAX_PORT, PORT_INC, PORT_PAGE_INC ),
        m_apply_button( ::Gtk::Stock::APPLY )
    {
      this->setup_widget();
    }

    PortEntry::~PortEntry() {}

    IPAddressInterface * PortEntry::address()
    {
      return m_address;
    }

    void PortEntry::set_address( IPAddressInterface* addr, unsigned lookup )
    {
      if ( m_conn_address_port ) m_conn_address_port.disconnect();
      m_lookup = lookup;
      m_address = addr;
      if ( m_address == NULL ) return;
      m_conn_address_port = m_address->signal_port_changed().connect ( sigc::mem_fun ( *this, &PortEntry::on_controlled_port_changed ) );
      this->on_controlled_port_changed();
    }
    
    void PortEntry::set_address( IPAddressInterface::pointer addr, unsigned lookup )
    {
      this->set_address( addr.get(), lookup );
    }
    
    void PortEntry::set_address ( IPv4::Address & addr, unsigned lookup )
    {
      this->set_address( &addr, lookup );
    }

    void PortEntry::set_address ( IPv6::Address & addr, unsigned lookup )
    {
      this->set_address( &addr, lookup );
    }

    void PortEntry::unset_address()
    {
      if ( m_conn_address_port ) m_conn_address_port.disconnect();
      m_lookup = LOOKUP_NONE;
      m_address = NULL;
      this->set_port ( 0 );
    }

    ::Gtk::SpinButton & PortEntry::port_widget( )
    {
      return m_port_widget;
    }

    ::Gtk::Adjustment & PortEntry::port_widget_adjustment( )
    {
      return m_port_widget_adjustment;
    }

    uint32_t PortEntry::port( )
    {
      return m_port_widget.get_value_as_int();
    }

    void PortEntry::set_port ( uint32_t p )
    {
      m_port_widget.set_value ( p );
    }

    std::string PortEntry::servicename()
    {
      uint32_t p = this->port();

      std::string sn;

      if ( m_lookup & LOOKUP_IPV4 ) sn = IPv4::service_to_servicename ( p );

      if ( not sn.empty() ) return sn;

      if ( m_lookup & LOOKUP_IPV6 ) sn = IPv6::service_to_servicename ( p );

      if ( not sn.empty() ) return sn;

      return "unknown";
    }

    bool PortEntry::is_servicename_label_visible()
    {
      return m_servicename_label.is_visible();
    }

    void PortEntry::show_servicename_label ( bool show )
    {
      if ( show )
        m_servicename_label.show_all();
      else
        m_servicename_label.hide();
    }

    ::Gtk::Button & PortEntry::apply_button()
    {
      return m_apply_button;
    }

    bool PortEntry::is_immediate_apply_enabled()
    {
      return m_immediate_apply;
    }

    void PortEntry::enable_immediate_apply ( bool enable )
    {
      m_immediate_apply = enable;

      if ( enable )
      {
        m_apply_button.hide();
        this->apply();
      }
      else
      {
        m_apply_button.show();
      }
    }

    void PortEntry::apply()
    {
      unsigned port = 0;
      std::stringstream oss;
      
      std::string porttext = m_port_widget.get_text();
      if ( porttext.size() > 0 )
      {
        m_conn_port_widget_changed.block();
        oss.str ( m_port_widget.get_text() );
        oss >> port;
        m_port_widget.set_value ( port );
        m_servicename_label.set_markup ( this->servicename() );
        m_apply_button.set_sensitive(false);
        if ( m_address ) m_address->set_port( port );
        m_conn_port_widget_changed.unblock();
      }
    }

    void PortEntry::on_controlled_port_changed( )
    {
      uint32_t port;

      m_conn_port_widget_changed.block();
      if ( m_address )
        port = m_address->port();
      else
        port = 0;
      m_port_widget.set_value(port);
      m_servicename_label.set_markup ( this->servicename() );
      m_apply_button.set_sensitive(false);
      m_conn_port_widget_changed.unblock();
    }

    void PortEntry::on_port_widget_changed( )
    {
      if ( m_immediate_apply ) this->apply();
      else m_apply_button.set_sensitive(true);
    }

    void PortEntry::on_spinbutton_text_entry ( const Glib::ustring & text, int * position )
    {
      if ( m_immediate_apply ) this->apply();
      else m_apply_button.set_sensitive(true);
    }

    void PortEntry::on_spinbutton_text_delete ( int start, int end )
    {
      if ( m_immediate_apply ) this->apply();
      else m_apply_button.set_sensitive(true);
    }

    void PortEntry::setup_widget( )
    {
      set_homogeneous ( false );

      ::Gtk::HBox* hbox = ::Gtk::manage ( new ::Gtk::HBox() );
      hbox->set_homogeneous ( false );

      hbox->pack_start ( m_port_widget );
      m_port_widget.configure ( m_port_widget_adjustment, 0.5, 0 );

      m_conn_port_widget_changed = m_port_widget.signal_changed().connect ( sigc::mem_fun ( *this, &PortEntry::on_port_widget_changed ) );
      m_port_widget.signal_insert_text().connect ( sigc::mem_fun ( *this, &PortEntry::on_spinbutton_text_entry ) );
      m_port_widget.signal_delete_text().connect ( sigc::mem_fun ( *this, &PortEntry::on_spinbutton_text_delete ) );

      m_apply_button.hide();
      m_apply_button.set_no_show_all();
      m_apply_button.set_sensitive(false);
      m_apply_button.signal_clicked().connect( sigc::mem_fun(*this, &PortEntry::apply) );
      
      hbox->pack_start( m_apply_button );

      this->pack_start ( *hbox );

      m_servicename_label.set_markup ( this->servicename() );

      this->pack_start ( m_servicename_label );
    }

  }

}
