/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "serialcombobox.h"

#include <iostream>
#include <sstream>

namespace Conexus
{
  namespace Gtk
  {

    SerialComboBox::SerialComboBox(bool include_virtuals, bool use_enumeration):
        m_include_virtuals(include_virtuals),
        m_use_enumeration(use_enumeration)
    {
      m_reftreemodel = ::Gtk::ListStore::create ( m_modelcolumns );
      set_model ( m_reftreemodel );
      pack_start ( m_modelcolumns.description );
      pack_start ( m_modelcolumns.device );

      if ( m_use_enumeration ) load_from_enumerated_serials();
    }


    SerialComboBox::~SerialComboBox()
    {
    }

    std::string SerialComboBox::device( )
    {
      Glib::ustring device;
      if ( get_active_row_number() <= 0 )
        return std::string();
      device = get_active()->operator[] ( m_modelcolumns.device );
      return device;
    }
    
    bool SerialComboBox::include_virtuals()
    {
      return m_include_virtuals;
    }

    void SerialComboBox::set_include_virtuals(bool set)
    {
      if ( set == m_include_virtuals ) return;
      m_include_virtuals = set;
      if ( m_use_enumeration ) this->load_from_enumerated_serials();
    }

    bool SerialComboBox::use_enumeration()
    {
      return m_use_enumeration;
    }

    void SerialComboBox::set_use_enumeration(bool set)
    {
      if ( set == m_use_enumeration ) return;
      if ( m_use_enumeration ) this->load_from_enumerated_serials();
    }
    
    void SerialComboBox::set_device ( const std::string& device )
    {
      ::Gtk::TreeModel::Row row;
      bool set = false;

      ::Gtk::TreeModel::iterator iter;
      for ( iter = m_reftreemodel->children().begin(); !set && iter != m_reftreemodel->children().end(); iter++ )
      {
        if ( ( *iter ) [m_modelcolumns.device] == device )
        {
          set_active ( iter );
          set = true;
        }
      }
      if ( !set && device.size() > 0 )
      {
        row = * ( m_reftreemodel->append() );
        row[m_modelcolumns.device] = device;
        set_active ( m_reftreemodel->children().end()-1 );
      }
    }

    void SerialComboBox::load_from_enumerated_serials( )
    {
      ::Gtk::TreeModel::Row row;
      
      m_reftreemodel->clear();

      row = * ( m_reftreemodel->append() );
      row[m_modelcolumns.description] = "No device selected";

      #ifdef CONEXUS_HAVE_UDEV
        Serial::Devices devices = Serial::enumerate_devices(m_include_virtuals);

        for ( size_t i=0; i < devices.size(); i++ )
        {
          row = * ( m_reftreemodel->append() );
          row[m_modelcolumns.description] = devices[i].name;
          row[m_modelcolumns.device] = devices[i].path;
        }
      #endif

      if ( get_active_row_number() == -1 ) set_active ( 0 );
    }

  }

}

