/***************************************************************************
 *   Copyright (C) 2009 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "ipv6_udplite.h"

#ifdef IPPROTO_UDPLITE

#ifndef UDPLITE_SEND_CSCOV
#define UDPLITE_SEND_CSCOV  10
#endif

#ifndef UDPLITE_RECV_CSCOV
#define UDPLITE_RECV_CSCOV  11
#endif

namespace Conexus
{
  namespace IPv6
  {

    UDPLite::UDPLite( const struct in6_addr& local_if, uint16_t localport, uint16_t remoteport ) :
        UDP( local_if, localport, remoteport, IPPROTO_UDPLITE )
    {
    }

    UDPLite::UDPLite( const std::string& local_if, uint16_t localport, uint16_t remoteport ) :
        UDP( local_if, localport, remoteport, IPPROTO_UDPLITE )
    {
    }

    UDPLite::pointer UDPLite::create( uint16_t localport, uint16_t remoteport )
    {
      return pointer( new UDPLite( in6addr_any, localport, remoteport ) );
    }

    UDPLite::pointer UDPLite::create( const struct in6_addr& local_if, uint16_t localport, uint16_t remoteport )
    {
      return pointer( new UDPLite( local_if, localport, remoteport ) );
    }

    UDPLite::pointer UDPLite::create( const std::string & local_if, uint16_t localport, uint16_t remoteport )
    {
      return pointer( new UDPLite( local_if, localport, remoteport ) );
    }

    UDPLite::~UDPLite() throw()
    {
    }

    uint16_t UDPLite::send_coverage()
    {
      int coverage;
      Socket::option(IPPROTO_UDPLITE, UDPLITE_SEND_CSCOV, coverage, ENDPOINT_OPENED);
      return static_cast<uint16_t>(coverage);
    }

    uint16_t UDPLite::set_send_coverage(uint16_t coverage_octets)
    {
      int coverage = coverage_octets;
      Socket::set_option(IPPROTO_UDPLITE, UDPLITE_SEND_CSCOV, coverage, ENDPOINT_OPENED);
      return this->send_coverage();
    }

    uint16_t UDPLite::set_payload_send_coverage(uint16_t coverage_octets)
    {
      int coverage;
      if ( coverage_octets >= 65527 /*2^16-1-8*/) coverage = 0;
      else coverage = coverage_octets;
      Socket::set_option(IPPROTO_UDPLITE, UDPLITE_SEND_CSCOV, coverage, ENDPOINT_OPENED);
      return this->send_coverage();
    }

    uint16_t UDPLite::recv_coverage()
    {
      int coverage;
      Socket::option(IPPROTO_UDPLITE, UDPLITE_RECV_CSCOV, coverage, ENDPOINT_OPENED);
      return coverage;
    }

    uint16_t UDPLite::set_recv_coverage(uint16_t coverage_octets)
    {
      int coverage = coverage_octets;
      Socket::set_option(IPPROTO_UDPLITE, UDPLITE_RECV_CSCOV, coverage, ENDPOINT_OPENED);
      return this->recv_coverage();
    }

    uint16_t UDPLite::set_payload_recv_coverage(uint16_t coverage_octets)
    {
      int coverage;
      if ( coverage_octets >= 65527 /*2^16-1-8*/) coverage = 0;
      else coverage = coverage_octets;
      Socket::set_option(IPPROTO_UDPLITE, UDPLITE_RECV_CSCOV, coverage, ENDPOINT_OPENED);
      return this->recv_coverage();
    }

  }
}

#endif

