/***************************************************************************
 *   Copyright (C) 2001,2002,2003,2004,2005,2006,2007,2008,2009            *
 *   by Rick L. Vinyard, Jr.                                               *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "utility.h"

#include <glibmm/thread.h>

#include <conexus/file.h>
#include <conexus/ipv4_tcp.h>
#include <conexus/ipv4_tcpserver.h>
#include <conexus/ipv4_udpposet.h>
#include <conexus/ipv4_udp.h>
#include <conexus/ipv6_tcp.h>
#include <conexus/ipv6_tcpserver.h>
#include <conexus/ipv6_udp.h>
#include <conexus/ipv6_udpposet.h>
#include <conexus/ll_packet.h>
#include <conexus/messagequeue.h>
#include <conexus/pipe.h>
#include <conexus/serial.h>
#include <conexus/tty.h>

#ifdef CONEXUS_HAVE_DBUS
  #include <dbus-cxx/utility.h>
#endif

namespace Conexus
{

  bool __use_dispatcher_default = false;
  bool __initialized = false;
  Factory::pointer __default_factory(Factory::create());
  Registry::pointer __default_registry(Registry::create());


  void init( bool use_dispatcher_default, bool init_dbus )
  {

    __use_dispatcher_default = use_dispatcher_default;
    if ( !Glib::thread_supported() )
      Glib::thread_init();

    __initialized = true;

    Factory& factory = default_factory();

    factory.add( "File", File::generic_create );
    factory.add( "IPv4::TCP", IPv4::TCP::generic_create );
    factory.add( "IPv4::TCPServer", IPv4::TCPServer::generic_create );
    factory.add( "IPv4::UDP", IPv4::UDP::generic_create );
    factory.add( "IPv4::UDPPoset", IPv4::UDPPoset::generic_create );
    factory.add( "IPv6::TCP", IPv4::TCP::generic_create );
    factory.add( "IPv6::TCPServer", IPv4::TCPServer::generic_create );
    factory.add( "IPv6::UDP", IPv4::UDP::generic_create );
    factory.add( "IPv6::UDPPoset", IPv4::UDPPoset::generic_create );
    factory.add( "Packet", LL::Packet::generic_create );
    factory.add( "MessageQueue", MessageQueue::generic_create );
    factory.add( "Pipe", Pipe::generic_create );
    factory.add( "Serial", Serial::generic_create );
    factory.add( "TTY", TTY::generic_create );

    #ifdef CONEXUS_HAVE_DBUS
    if ( init_dbus ) DBus::init();
    #endif
  }

  bool initialized()
  {
    return __initialized;
  }

  bool use_dispatcher_default( )
  {
    return __use_dispatcher_default;
  }

  void set_use_dispatcher_default( bool use_dispatcher_default )
  {
    __use_dispatcher_default = use_dispatcher_default;
  }

  void set_default_multicast_hop_limit( unsigned hop_limit )
  {
    IPv4::UDP::set_default_multicast_hop_limit( hop_limit );
    IPv6::UDP::set_default_multicast_hop_limit( hop_limit );
  }

  void enable_default_multicast_loopback( bool enable )
  {
    IPv4::UDP::enable_default_multicast_loopback( enable );
    IPv6::UDP::enable_default_multicast_loopback( enable );
  }

  Factory& default_factory( )
  {
    return *__default_factory;
  }

  bool set_default_factory( Factory::pointer new_factory )
  {
    if ( ! new_factory )
      return false;
    __default_factory = new_factory;
    return true;
  }

  Registry & default_registry( )
  {
    return *__default_registry;
  }

  bool set_default_registry( Registry::pointer new_registry )
  {
    if ( ! new_registry )
      return false;
    __default_registry = new_registry;
    return true;
  }

}

